from os import system as syst
import sys
import requests

def gdrive_linux(link, name):
  if name == "":
    print("ERROR - You haven't specified file name")
    sys.exit(0)

  FILEID = f'https://docs.google.com/uc?export=download&id={link}'
  FILENAME = name
  syst(f'wget --load-cookies /tmp/cookies.txt "https://docs.google.com/uc?export=download&confirm=$(wget --quiet --save-cookies /tmp/cookies.txt --keep-session-cookies --no-check-certificate {FILEID} -O- | sed -rn \'s/.*confirm=([0-9A-Za-z_]+).*/\1\n/p\')&id={link}" -O {FILENAME} && rm -rf /tmp/cookies.txt')

def gdrive(id, destination):
    def get_confirm_token(response):
        for key, value in response.cookies.items():
            if key.startswith('download_warning'):
                return value

        return None

    def save_response_content(response, destination):
        CHUNK_SIZE = 32768

        with open(destination, "wb") as f:
            for chunk in response.iter_content(CHUNK_SIZE):
                if chunk: # filter out keep-alive new chunks
                    f.write(chunk)

    URL = "https://docs.google.com/uc?export=download"

    session = requests.Session()

    response = session.get(URL, params = { 'id' : id }, stream = True)
    token = get_confirm_token(response)

    if token:
        params = { 'id' : id, 'confirm' : token }
        response = session.get(URL, params = params, stream = True)

    save_response_content(response, destination)


import requests , os , sys
from lxml import html

RPGURL_LOGIN = "https://rapidgator.net/api/v2/user/login"
#RPGURL_FILEINFO = "https://rapidgator.net/api/v2/file/info/"
RPGURL_FILEDOWNLOAD = "https://rapidgator.net/api/v2/file/download/"
RPG_XPATH_FILENAME = "//div[@class='text-block file-descr']//p//a//text()"


#USERNAME = sys.argv[1]
#PASSWORD = sys.argv[2]
#PATH = sys.argv[4]
#RAPIDURL = sys.argv[3]

def rapidgator(USERNAME, PASSWORD, PATH, RAPIDURL):
    
    #Validate
    if not os.path.exists(PATH) and os.path.isdir(PATH) :
        print ("Save Directory not exist")
        sys.exit(0)


    PARAMS = {"login": USERNAME, "password": PASSWORD}

    r = requests.get(url=RPGURL_LOGIN, params=PARAMS)
    data = r.json()
    print("data : " + str(data))
    token = data["response"]["token"]


    file_name = None

    _file_id = RAPIDURL.split("/")[4]

## If want assign filename from URL (htttp://rg.to/408320/assign_filename_by_yourself.rar ==> file_name = assign_filename_by_yourself.rar
# try:
#     file_name = _url.split("/")[5]
#     file_name = file_name.replace(".html", "")
#     print("expect file name : " + file_name)
# except:
#     print("file name not found")
#     file_name = str(int(round(time.time() * 1000)))
## end If

## If want get filename from original uploader
    page = requests.get(RAPIDURL)
    print(page.status_code)
    if page.status_code > 400 :
        print("URL : "+RAPIDURL+ "get HTTP STATUS "+str(page.status_code))
        sys.exit(0)
    else :
        root = html.fromstring(page.text)
        tree = root.getroottree()
        result = root.xpath(RPG_XPATH_FILENAME)
        file_name = ''.join([word.strip() for word in result])
        file_name = len(file_name) > 0 and file_name or _file_id
## end IF

    url = RPGURL_FILEDOWNLOAD
    params = {"file_id": _file_id, "token": token}
    r = requests.get(url=url, params=params)
    data = r.json()
    print(data)
    _dl_url = data["response"]["download_url"]
    print("Response :" + str(data) + " \ngetDownloadLink : " + str(_dl_url))
    download_cmd = "wget -P " + PATH + " -O '" +PATH + "/" + file_name + "' '" + _dl_url + "'"

    print(download_cmd)
    os.system(download_cmd)



import requests
from bs4 import BeautifulSoup
from os.path import join
from os import makedirs
import sys


def onechapter(web, output_dir):
    res = requests.get(web)
    html_content = res.text
    soup = BeautifulSoup(html_content, 'html.parser')
    img_tags = soup.find_all('img')
    #print(img_tags)
    parts = web.split("/")
    print(parts[4],parts[5])
    folder = f'{output_dir}\{parts[4]}\{parts[5]}'
    makedirs(folder, exist_ok=True)
    for x in range(1,len(img_tags)+1):
        if 1 <= x <= 9:
            downlink = f'https://img.imgxyzz.xyz/{parts[4]}/{parts[5]}/00{x}.jpg'
        elif x > 9:
            downlink = f'https://img.imgxyzz.xyz/{parts[4]}/{parts[5]}/0{x}.jpg'
        print(downlink)
        image_save_path = join(folder, f'0{x}.jpg')
        res = requests.get(downlink)
        with open(image_save_path, 'wb') as f:
            f.write(res.content)
    '''
    res = requests.get(web)
    html_content = res.text
    soup = BeautifulSoup(html_content, 'html.parser')
    img_tags = soup.find_all('img')
    image_links = [img['data-src'] for img in img_tags]
    for i, link in enumerate(image_links):
        image_save_path = join(output_dir, f'image_{i}.jpg')
        res = requests.get(link)
        with open(image_save_path, 'wb') as f:
            f.write(res.content)
    '''

def multichapters(link, dir, from_chapter, latest_chapter, *special_chapters):
    listchaps = []
    for c in range(from_chapter,latest_chapter+1):
        chapterlink = f'{link}chapter-{c}'
        listchaps.append(chapterlink)
        if special_chapters != 0:
            for x in special_chapters:
                c = c + x
                specialchapterlink = f'{link}chapter-{c}'
                c = c - x
                listchaps.append(specialchapterlink)
    for x in listchaps:
        onechapter(x, dir)

#onechapter("https://khotruyen.cc/truyen-tranh/ba-chi-chu-nha-the-owner-of-a-building/chapter-115/","\downloads")
multichapters("https://khotruyen.cc/truyen-tranh/ba-chi-chu-nha-the-owner-of-a-building/chapter-115/","\downloads",110,115,0)


#Code from: https://github.com/xiatg/iwara-python-api/blob/main/api_client.py
#Edited by @IAmNamNguyen

import requests, hashlib, os
import json
import sys
#email = sys.argv[1]
#password = sys.argv[2]
#video_id = sys.argv[3]
#specify_name = sys.argv[4]
#preset_name = sys.argv[5]


class BearerAuth(requests.auth.AuthBase):
    """Bearer Authentication"""
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['Authorization'] = 'Bearer ' + self.token
        return r

class ApiClient:
    api_url = 'https://api.iwara.tv'
    file_url = 'https://files.iwara.tv'
    def __init__(self, email, password, api_url, file_url):
        self.email = email
        self.password = password

        # self.headers = {
        # 'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.181 Safari/537.36',
        # 'X-Version': 's'
        # }

        # API
        self.api_url = api_url
        self.file_url = file_url
        self.timeout = 30
        # self.max_retries = 5
        self.download_timeout = 300
        self.token = None

        # HTML
        # self.html_url = html_url

        # Cloudscraper
        # self.scraper = cloudscraper.create_scraper(browser={'browser': 'firefox','platform': 'windows','mobile': False}, 
        #                                         #    interpreter = 'nodejs'
        #                                         )
        # Requests-html
        # self.session = HTMLSession()

    def login(self) -> requests.Response:
        url = self.api_url + '/user/login'
        json = {'email': self.email, 'password': self.password}
        r = requests.post(url, json=json, timeout=self.timeout)
        try:
            self.token = r.json()['token']
            print('API Login success')
        except:
            print('API Login failed')

        # try:
        #     # Cloudscraper
        #     # r = self.scraper.post(url, json=json, headers=self.headers, timeout=self.timeout)

        #     # Requests-html
        #     r = self.session.post(url, json=json, headers=self.headers, timeout=self.timeout)
        # except:
        #     print('BS4 Login failed')

        return r

    # limit query is not working
    def get_videos(self, sort = 'date', rating = 'all', page = 0, limit = 32, subscribed = False) -> requests.Response:
        """# Get new videos from iwara.tv
        - sort: date, trending, popularity, views, likes
        - rating: all, general, ecchi
        """
        url = self.api_url + '/videos'
        params = {'sort': sort, 
                  'rating': rating, 
                  'page': page, 
                  'limit': limit,
                  'subscribed': 'true' if subscribed else 'false',
                  }
        if self.token is None:
            r = requests.get(url, params=params, timeout=self.timeout)
        else:

            # Verbose Debug
            # request = requests.Request('GET', url, params=params, auth=BearerAuth(self.token))
            # print(request.prepare().method, request.prepare().url, request.prepare().headers, request.prepare().body, sep='\n')
            # r = requests.Session().send(request.prepare())

            r = requests.get(url, params=params, auth=BearerAuth(self.token), timeout=self.timeout)

        #Debug
        print("[DEBUG] get_videos response:", r)

        return r
    
    def get_video(self, video_id) -> requests.Response:
        """# Get video info from iwara.tv
        """
        url = self.api_url + '/video/' + video_id

        if self.token is None:
            r = requests.get(url, timeout=self.timeout)
        else:
            r = requests.get(url, auth=BearerAuth(self.token), timeout=self.timeout)

        #Debug
        print("[DEBUG] get_video response:", r)

        return r
    
    def download_video_thumbnail(self, video_id) -> str:
        """# Download video thumbnail from iwara.tv
        """
        video = self.get_video(video_id).json()

        file_id = video['file']['id']
        thumbnail_id = video['thumbnail']
        
        url = self.file_url + '/image/original/' + file_id + '/thumbnail-{:02d}.jpg'.format(thumbnail_id)

        thumbnail_file_name = video_id + '.jpg'

        if (os.path.exists(thumbnail_file_name)):
            print(f"Video ID {video_id} thumbnail already downloaded, skipped downloading. ")
            return thumbnail_file_name
        
        print(f"Downloading thumbnail for video ID: {video_id} ...")
        with open(thumbnail_file_name, "wb") as f:
            for chunk in requests.get(url).iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)
                    f.flush()

        return thumbnail_file_name

    def download_video(self, video_id, specify_name, preset_name) -> str:
        """# Download video from iwara.tv
        """
        if specify_name:
          if preset_name == "":
            print("ERROR - You haven't specified file name")
            sys.exit(0)
        

        # html
        # url = self.html_url + '/video/' + video_id

        # Cloudscraer
        # html = self.scraper.get(url, auth=BearerAuth(self.token), timeout=self.timeout).text

        # Requests-html
        # html = self.session.get(url, auth=BearerAuth(self.token), timeout=self.timeout).text

        # print(html)
        # html = BeautifulSoup(, 'html.parser')
        # downloadLink = html.find('div', class_='dropdown_content')
        # print(downloadLink)

        # API
        try:
            video = self.get_video(video_id).json()
        except Exception as e:
            raise Exception(f"Failed to get video info for video ID: {video_id}, error: {e}")

        #Debug
        print(video)

        #Chuyển JSON thành dạng Python đọc được
        python_readable_json = json.loads(video)

        url = video['fileUrl']
        file_id = video['file']['id']
        expires = url.split('/')[4].split('?')[1].split('&')[0].split('=')[1]

        # IMPORTANT: This might change in the future.
        SHA_postfix = "_5nFp9kmbNnHdAFhaqMvt"

        SHA_key = file_id + "_" + expires + SHA_postfix
        hash = hashlib.sha1(SHA_key.encode('utf-8')).hexdigest()

        headers = {"X-Version": hash}

        resources = requests.get(url, headers=headers, auth=BearerAuth(self.token), timeout=self.timeout).json()
        
        #Debug
        print(resources)

        resources_by_quality = [None for i in range(10)]

        for resource in resources:
            if resource['name'] == 'Source':
                resources_by_quality[0] = resource
            # elif resource['name'] == '1080':
            #     resources_by_quality[1] = resource
            # elif resource['name'] == '720':
            #     resources_by_quality[2] = resource
            # elif resource['name'] == '480':
            #     resources_by_quality[3] = resource
            # elif resource['name'] == '540':
                # resources_by_quality[4] = resource
            # elif resource['name'] == '360':
                # resources_by_quality[5] = resource

        for resource in resources_by_quality:
            if resource is not None:
                #Debug
                print(resource)

                download_link = "https:" + resource['src']['download']
                file_type = resource['type'].split('/')[1]
                #Tải với tên định trước
                if specify_name:
                    video_file_name = preset_name + '.' + file_type
                else:
                    video_file_name = python_readable_json['title'] + '.' + file_type

                if (os.path.exists(video_file_name)):
                    print(f"Video ID {video_id} Already downloaded, skipped downloading. ")
                    return video_file_name

                print(f"Downloading video ID: {video_id} ...")
                try:
                    with open(video_file_name, "wb") as f:
                        for chunk in requests.get(download_link).iter_content(chunk_size=1024):
                            if chunk:
                                f.write(chunk)
                                f.flush()
                    return video_file_name
                except Exception as e:
                    os.remove(video_file_name)
                    raise Exception(f"Failed to download video ID: {video_id}, error: {e}")

            
        raise Exception("No video with Source quality found")


#Template run
'''
client = ApiClient(email, password)
client.login()
client.download_video(video_id, specify_name, preset_name)
'''


from os import system as syst
from urllib.parse import urlparse as urlp
def directdown_linux(link, name, no_check_certificate):
  if name == '':
    path = urlp(link).path.split('/')
    name = path[len(path)-1]
    name = name.replace("%20","_")
    name = name.replace("%28","(")
    name = name.replace("%29",")")
  if no_check_certificate:
    syst(f'wget -O {name} {link} --no-check-certificate')
  else:
    syst(f'wget -O {name} {link}')


from setuptools import setup, find_packages

VERSION = '0.0.2' 
DESCRIPTION = 'Sites Media Downloader'
LONG_DESCRIPTION = 'My first Python package to download files from sites'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="namsmd", 
        version=VERSION,
        author="Nam Nguyen",
        author_email="nnamnguyenn@07kpq.onmicrosoft.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['lxml','beautifulsoup4'], # add any additional packages that 
        # needs to be installed along with your package. Eg: 'caer'
        
        keywords=['python', 'first package'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            #"Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            #"Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)


import requests
from bs4 import BeautifulSoup
from os.path import join
from os import makedirs
import sys

#https://stackoverflow.com/questions/14587728/what-does-this-error-in-beautiful-soup-means

def onechapter(web,output_dir):
    res = requests.get(web)
    html_content = res.text
    soup = BeautifulSoup(html_content, 'html.parser')
    img_tags = soup.findAll('img',{"data-src":True})
    image_links = [img['data-src'] for img in img_tags]
    #['https:', '', 'evvdsfgefdszihfdx.hentaivn.lat', 'HGHrrFSDweqZSfd', 'lgbdfexijzbxsdrl', '2024', '01', '12', '1705045067-0.png?imgmax=1200']
    #https://i3.hhentai.net/images/5/6/7/8
    parts = web.split("/")
    print(parts[2],parts[3])
    folder = f'{output_dir}\{parts[2]}\{parts[3]}'
    makedirs(folder, exist_ok=True)
    for x in image_links:
        l = x.split("/")
        name = l[8].replace("?imgmax=1200","")
        link = f'https://i3.hhentai.net/images/{l[5]}/{l[6]}/{l[7]}/{l[8]}'
        image_save_path = join(folder, name)
        print(image_save_path)
        res = requests.get(link)
        with open(image_save_path, 'wb') as f:
            f.write(res.content)
 

def multichapters(link, dir, from_chapter, latest_chapter, *special_chapters):
    listchaps = []
    for c in range(from_chapter,latest_chapter+1):
        chapterlink = f'{link}chapter-{c}'
        listchaps.append(chapterlink)
        if special_chapters != 0:
            for x in special_chapters:
                c = c + x
                specialchapterlink = f'{link}chapter-{c}'
                c = c - x
                listchaps.append(specialchapterlink)
    for x in listchaps:
        onechapter(x, dir)

onechapter("https://hentaivn.lat/36055-67235-xem-truyen-goblin-kara-hajimeru-sekai-seifuku-chap-1.html","\downloads")
#multichapters("https://khotruyen.cc/truyen-tranh/ba-chi-chu-nha-the-owner-of-a-building/chapter-115/","\downloads",110,115,0)